let _ = GMain.init ()

(* _____________ Bouton About ________________ *)

let show () =
  let dialog =
    GWindow.about_dialog
      ~name:"OCRodile"
      ~version:"1.0"
      ~authors:["Durieux Edouard";
		"Florent Nicolas";
                "Perronin Thomas";
		"Vinckenbosch Thomas"]
      ~icon_name:"logo.png"
      ~website:"http://www.ocrodile.fr/"
      ~position:`CENTER_ON_PARENT
      ~destroy_with_parent:true () in
    dialog#set_logo(GdkPixbuf.from_file "img/logo.png");
    ignore (dialog#connect#response
              ~callback:(fun _ -> dialog#show ()));
    ignore(dialog#run ());
    dialog#misc#hide ()

let img_path = ref "init"

let step1_clicked = ref false

(*______________________Interface ___________________________________________*)

let main() =


    if (Array.length Sys.argv) > 1 && Sys.argv.(1) = "-help" then
        exit (Sys.command "cat README")
    else

(* _______________ Fenêtre principale ________ *)

let window = GWindow.window
    ~title:"OCRodile ----------- OCR"
    ~position:`CENTER
    ~resizable:false
    ~width:1200
    ~height:800
    ~border_width:10 () in
ignore(window#connect#destroy ~callback:GMain.Main.quit);

(* box principale contenant tout le reste *)
let major_box = GPack.vbox
    ~border_width:0
    ~packing:window#add() in

let main_menu = GPack.hbox
   ~height:75
   ~packing:major_box#add() in

let main_box = GPack.hbox
    ~border_width:0
    ~packing:major_box#add() in

(* box de gauche avec les boutons de traitement et menus *)
let left_box = GPack.vbox
    ~height:100
    ~border_width:5
    ~packing:main_box#add() in

(* Box de droite avec affichage de l'image *)
let frame = GBin.frame
    ~label:"Image"
    ~width:600
    ~border_width:5
    ~packing:main_box#add() in

(*  Définition de la partie droite avec une image scrollable *)
let scroll = GBin.scrolled_window
    ~height:600
    ~hpolicy:`ALWAYS
    ~vpolicy:`ALWAYS
    ~packing:frame#add () in

let picture_area  = GMisc.image
    ~file:"img/logo.png"
    ~packing:scroll#add_with_viewport () in

(* _________________ MENU _____________________ *)


let menu_box = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~spacing:10
  ~packing:(main_menu#pack ~expand:false) () in

let copy_ref path =
	img_path := path in

(* OPEN FILE FILTER *)
let filter () = GFile.filter
	~name:"Images"
	~patterns:[ "*.bmp" ; "*.jpg" ; "*.png"] () in

(* ____________________________ Menu de prétraitement __________ *)

(* fonctions associées au prétraitement *)

let greylevel () =
    let img = (Sdlloader.load_image (!img_path)) in
    Sdlvideo.save_BMP
    (Matrix.tab2image (Matrix.to3coord ((Filter.process
    (Matrix.to1coord (Matrix.image2tab img)) Filter.grey))))
    "file/finalimage";img_path:="file/finalimage";
    picture_area#set_file ((Sys.getcwd ())^"/file/finalimage")
in

let binarisation () =
    let img = (Sdlloader.load_image (!img_path)) in
    Sdlvideo.save_BMP (Matrix.tab2image
        (Pretreatments.toBlack (Matrix.image2tab img))) "file/finalimage";
    picture_area#set_file ((Sys.getcwd ())^"/file/finalimage")
in

let rotation () =
    let img = (Sdlloader.load_image (!img_path)) in
    let mat_img = Matrix.image2tab img in
    Sdlvideo.save_BMP (Matrix.tab2image
        (Rotate.rotate mat_img (-.Rotate.findang
                            (Matrix.to1coord mat_img)))) "file/finalimage";
    picture_area#set_file ((Sys.getcwd ())^"/file/finalimage")

in

let segmentation_line () =
    let img = (Sdlloader.load_image (!img_path)) in
    let mat_img = Matrix.to1coord (Matrix.image2tab img)
    in TextProcess.print_lines mat_img;
    let mat_final = Matrix.to3coord mat_img in
    Sdlvideo.save_BMP (Matrix.tab2image mat_final) "file/finalimage";
    picture_area#set_file ((Sys.getcwd ())^"/file/finalimage")
in

let segmentation_col () =
    let img = (Sdlloader.load_image (!img_path)) in
    let mat_img = Matrix.to1coord (Matrix.image2tab img)
    in TextProcess.print_columns_call mat_img
    (TextProcess.search_lines_v2 mat_img);
    let mat_final = Matrix.to3coord mat_img in
    Sdlvideo.save_BMP (Matrix.tab2image mat_final) "file/finalimage";
    picture_area#set_file ((Sys.getcwd ())^"/file/finalimage")
in

let box_2D = GPack.vbox
    ~width:600
    ~packing:left_box#add() in

(* définition de la fenêtre de prétraitement sous le menu de base *)
let pre_processing_frame = GBin.frame
  ~label:"Pretreatment"
  ~height:50
  ~packing:box_2D#add() in

let hbox = GPack.hbox
  ~border_width:0
  ~packing:pre_processing_frame#add() in

(* fenêtre de segmentation *)
let segmentation_frame = GBin.frame
  ~label:"Segmentation / Text recognition"
  ~height:50
  ~packing:box_2D#add() in

let hbox2 = GPack.hbox
  ~border_width:0
  ~packing:segmentation_frame#add() in

(* 3 box/boutons pour les 3 étapes de prétraitement *)
let box1 = GPack.vbox
  ~border_width:0
  ~packing:hbox#add() in

let box2 = GPack.vbox
  ~border_width:0
  ~packing:hbox#add() in

let box3 = GPack.vbox
   ~border_width:0
   ~packing:hbox#add() in

(* 2 box/boutons pour la segmentation *)
let box4 = GPack.vbox
   ~border_width:0
   ~packing:hbox2#add() in

let box5 = GPack.vbox
   ~border_width:0
   ~packing:hbox2#add() in

(* boutons à proprement parler *)
(* bouton de niveau de gris *)
let bbox1 = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~height: 75
  ~spacing:10
  ~packing:(box1#pack ~expand:false) () in

let greylevel_button = GButton.button
  ~label:"Grey level"
  ~packing:bbox1#add () in
ignore (greylevel_button#connect#clicked ~callback:(greylevel));

(* bouton binarisation / avec amélioration filtre *)
let bbox2 = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~height: 75
  ~spacing:10
  ~packing:(box2#pack ~expand:false) () in

let binarisation_button = GButton.button
  ~label:"Binarisation"
  ~packing:bbox2#add () in
   ignore (binarisation_button#connect#clicked ~callback:(binarisation));

(* bouton rotation *)
let bbox3 = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~height: 75
  ~spacing:10
  ~packing:(box3#pack ~expand:false) () in

let rotation_button = GButton.button
  ~label:"Rotation"
  ~packing:bbox3#add () in
   ignore (rotation_button#connect#clicked ~callback:(rotation));

(* boutons segmentation *)
let bbox4 = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~height: 75
  ~spacing:10
  ~packing:(box4#pack ~expand:false) () in

let segmentation_button = GButton.button
  ~label:"Line detection"
  ~packing:bbox4#add () in
   ignore (segmentation_button#connect#clicked ~callback:(segmentation_line));

let bbox5 = GPack.button_box `HORIZONTAL
  ~layout:`SPREAD
  ~height: 75
  ~spacing:10
  ~packing:(box5#pack ~expand:false) () in

let segmentation_button = GButton.button
  ~label:"Column detection"
  ~packing:bbox5#add () in
   ignore (segmentation_button#connect#clicked ~callback:(segmentation_col));

(* _________ Text recognition je sais pas encore ce que c'est que ce truc *)

let box_3D = GPack.hbox
  ~border_width:0
  ~height:300
  ~packing:left_box#add() in

let scroll = GBin.scrolled_window
      ~hpolicy:`AUTOMATIC
      ~vpolicy:`AUTOMATIC
      ~shadow_type:`IN
      ~packing:(box_3D#pack ~expand:true) () in

let textbox = GText.view
  ~packing:scroll#add () in
GtkSpell.attach ~lang:"en_FR" textbox;
   textbox#buffer#set_text("roj 'ej mlp");


(*let save file value =
   let och = open_out file in
   output_value och value;
   close_out och in*)

let nerd () = ()
         (* let img = (Sdlloader.load_image
                  ((Sys.getcwd ())^"/TMP/Result")) in
         let (w,h) = (get_dims img) in
         let img_matrix = Array.make_matrix w h 0 in
         surface_to_matrix img img_matrix;
	 let red = (Neuron.test_reduct img_matrix w 16) in
	 save ((Sys.getcwd ())^"/TMP/Mat") red;
	 let network = (new Neuron.neuron_network 26) in
         network#init;
         textbox#buffer#set_text (network#nerd_process red) *) in

let box3 = GPack.vbox
    ~border_width:0
    ~packing:hbox2#add() in

 let bbox3 = GPack.button_box `HORIZONTAL
    ~layout:`SPREAD
    ~height: 75
    ~spacing:10
    ~packing:(box3#pack ~expand:false) () in

 let reconnaissance_button = GButton.button
    ~label:"Recognition"
    ~packing:bbox3#add () in
     ignore (reconnaissance_button#connect#clicked
    ~callback:(nerd););

(* _____________ Definition des boutons du menu _____________________________*)

let touch_picture btn () =
  Gaux.may copy_ref btn#filename;
  picture_area#set_file (!img_path);
  step1_clicked := false in

let open_b = GFile.chooser_button
		~action: `OPEN
		~width:100
		~height:25
		~packing:(menu_box#add) () in
  open_b#set_filter (filter ());
  ignore(open_b#connect#selection_changed (touch_picture open_b););

let reset () =
	picture_area#set_file ((Sys.getcwd ())^"img/logo.png");
        copy_ref "init.png";
	step1_clicked:= false;
	img_path := "" in

let reset_button = GButton.button
  ~label:"Reset"
  ~packing:menu_box#add () in
  ignore (reset_button#connect#clicked ~callback:(reset));


let about_button = GButton.button
  ~label:"About"
  ~stock: `ABOUT
  ~packing:menu_box#add ()in ignore (about_button#connect#clicked
  ~callback:(show));
  ignore(GMisc.image ~stock:`ABOUT ~packing:about_button#set_image ());


let close_button = GButton.button
  ~label:"Quit"
  ~packing:menu_box#add () in
  ignore (close_button#connect#clicked ~callback:(GMain.quit));


  (* ______________________ Fonction d'apel _________________*)
window#show();
GMain.Main.main ()

let _ = main ()
