(*----------------------------------------------------------------------------
* GET_DIMS
* ----------------------------------------------------------------------------
* DESCRIPTION : Returns the dimensions of an SDLsurface
*
* TYPE : Function, returns (x,y)
*
* ARGUMENTS :
* img : Source SDLsurface
* ---------------------------------------------------------------------------*)

let get_dims img =
  ((Sdlvideo.surface_info img).Sdlvideo.w,
  (Sdlvideo.surface_info img).Sdlvideo.h)

(* ---------------------------------------------------------------------------
* TAB2IMAGE
* ---------------------------------------------------------------------------
* DESCRIPTION : tab2image produces an image from an array
*
* TYPE : Function, returns a SDLvideo.Surface
*
* ARGUMENTS :
* tab : (int*int*int) array array
* ---------------------------------------------------------------------------*)

let tab2image tab =
    let img =
    (Sdlvideo.create_RGB_surface [] (Array.length tab)
        (Array.length tab.(0))
        24 Int32.zero Int32.zero Int32.zero Int32.zero) in
            for i = 0 to (Array.length tab - 1) do
                for j = 0 to (Array.length tab.(i) - 1) do
                begin
                    Sdlvideo.put_pixel_color img i j tab.(i).(j)
                end;
                done;
            done;
    img
(* ---------------------------------------------------------------------------
* IMAGE2TAB
* ----------------------------------------------------------------------------
* DESCRIPTION : tab2image produces an array from an image
*
* TYPE : Function, returns an (int*int*int) array array
*
* ARGUMENTS :
* img : SDLvideo.surface
* ---------------------------------------------------------------------------*)

let image2tab img =
    let (width,height) = get_dims img in
        let tab = Array.make_matrix width height (0,0,0) in
            for i = 0 to width - 1 do
                for j = 0 to height - 1 do
                begin
                    tab.(i).(j) <- Sdlvideo.get_pixel_color img i j
                end;
                done;
            done;
    tab

(* ---------------------------------------------------------------------------
* TO1COORD
* ----------------------------------------------------------------------------
* DESCRIPTION : transform an (int * int * int) matrix into an int matrix
*
* TYPE : Function, returns an int array array
*
* ARGUMENTS :
* matsrc : (int*int*int) matrix
* ---------------------------------------------------------------------------*)

let to1coord matsrc =
    let matsize_x = Array.length matsrc in
    let matsize_y = Array.length matsrc.(0) in

    let matout = Array.make_matrix matsize_x matsize_y 0 in

    begin

        for i = 0 to matsize_x - 1 do
            for j = 0 to matsize_y - 1 do
                let (x,y,z) = matsrc.(i).(j) in
                matout.(i).(j) <- x;
            done;
        done;

        matout;

    end

(* ---------------------------------------------------------------------------
* TO3COORD
* ----------------------------------------------------------------------------
* DESCRIPTION : transform an int matrix into an (int * int * int) matrix
*
* TYPE : Function, returns an (int*int*int) array array
*
* ARGUMENTS :
* matsrc : int matrix
* ---------------------------------------------------------------------------*)

let to3coord matsrc =
    let matsize_x = Array.length matsrc in
    let matsize_y = Array.length matsrc.(0) in

    let matout = Array.make_matrix matsize_x matsize_y (0,0,0) in

        for i = 0 to matsize_x - 1 do
            for j = 0 to matsize_y - 1 do
                matout.(i).(j) <- (matsrc.(i).(j),
                                  matsrc.(i).(j),
                                  matsrc.(i).(j));
            done;
        done;

     matout;
