(* Rotating an image by a given angle *)

let pi = 3.14159265
let degree_to_rad ang = (pi*.ang) /. 180.0
let rad_to_degree ang = (180.0*.ang) /. pi

let rotx x y center_x center_y angle =
            int_of_float ((cos angle) *. float (x - center_x)
                        +. (sin angle) *. float (y - center_y) +. float
                        (center_x))

let roty x y center_x center_y angle =
            int_of_float (-.(sin angle) *. float (x - center_x)
                        +. (cos angle) *. float (y - center_y) +. float
                        (center_y))

let rotate mat angle =
    let angle = angle *. 3.14159265 /. 180. in
    let w = Array.length mat and
    h = Array.length mat.(0) in
    let dest_w = int_of_float (float w *. (abs_float (cos angle))
    +. float h *. (abs_float (sin angle))) in
    let dest_h = int_of_float (float h *. (abs_float (cos
    angle))
    +. float w *. (abs_float (sin angle))) in
    let dest_mat = Array.make_matrix dest_w dest_h
    (255,255,255) in
    for dest_x = 0 to dest_w - 1 do
        for dest_y = 0 to dest_h - 1 do
            let x = rotx dest_x dest_y (dest_w / 2) (dest_h/2) angle
            and y = roty dest_x dest_y (dest_w / 2) (dest_h/2) angle
            in
                if x >= 0 && x<w && y>=0 && y<h then
                    dest_mat.(dest_x).(dest_y) <- mat.(x).(y);
        done
    done;
dest_mat

(* Détection de l'angle de rotation *)

(* table of the rotation's distance r *)
let maketabalign w h ang lspxbk =
  let diag = w+h+1 in
  let tab = Array.make diag 0 in
    for i=0 to (Array.length lspxbk)-1  do
      let (x,y)= lspxbk.(i) in
      let r = truncate
	(abs_float(float_of_int(x)*.(cos ang)
		   +.float_of_int(y)*.(sin ang))) in
	tab.(r) <- tab.(r)+1;
    done;
    tab

(* make the histogram of r values given to an angle *)

let makealign w h ang lspxbk =
  let align = ref 0 in
  let tab = maketabalign w h ang lspxbk in
  let temp = ref tab.(0) in
    for i=1 to (Array.length tab)-1 do
      let op = ((!temp) - tab.(i)) in
	align := !align + op*op;
	temp := tab.(i);
    done;
    !align

(* Give the angle with the highest r value *)

let maxalg ary =
  let max = ref (ary.(0))
  and ang = ref 0 in
    for i=1 to ((Array.length ary)-1) do
      if ary.(i) > (!max) then
	begin
	  max := ary.(i);
	  ang := i;
	end;
    done;
    if (!ang -15)<=1 then
      (!ang -15)
    else
      (!ang -16)

(* give the array of the black pixel *)

let givepxbkarray w h matbw =
  let listpxbk = ref [] in
    for x=0 to w-1 do
      for y=0 to h-1 do
	if (matbw.(x).(y)==0) then
	  begin
	    listpxbk := (x,y)::(!listpxbk)
	  end;
      done;
    done;
    (Array.of_list (!listpxbk))

(* trouve l'angle de rotation du text *)

let findang matbw =
  let w = Array.length matbw
  and h = Array.length matbw.(0) in
  let listpxbk = givepxbkarray w h matbw in
  let nbang = 31 in
  let angarray =  Array.make nbang 0 in
    for i=0 to nbang-1 do
      angarray.(i) <- (makealign
			 w
			 h
			 (degree_to_rad(float_of_int(i-15)))
			 (listpxbk)
		      );
    done;
    float_of_int(maxalg angarray)
