
(* ///////////////// BEGIN TOOLS ////////////////// *)



(* ---------------------------------------------------------------------------
* WIDTH
* ----------------------------------------------------------------------------
* DESCRIPTION : Returns the width of the matrix
*
* TYPE : Function. Returns an int
* ---------------------------------------------------------------------------*)

let width matrix = (Array.length matrix)

(* ---------------------------------------------------------------------------
* HEIGHT
* ----------------------------------------------------------------------------
* DESCRIPTION : Returns the height of the matrix
*
* TYPE : Function. Return an int
* ---------------------------------------------------------------------------*)

let height matrix = (Array.length matrix.(0))

(* ---------------------------------------------------------------------------
* SIZE
* ----------------------------------------------------------------------------
* DESCRIPTION : Returns the size of a matrix
*
* TYPE : Function. Returns an int matrix
* ---------------------------------------------------------------------------*)

let size matrix = (width matrix) * (height matrix)

(* ---------------------------------------------------------------------------
* IS_TRUE
* ----------------------------------------------------------------------------
* DESCRIPTION : Tells if all param are differents from -1
*
* TYPE : Function. Returns boolean
* ---------------------------------------------------------------------------*)

let isTrue x1 x2 y1 y2 =
    let isIt = ref false in
    if (x1 <> -1 && x2 <> -1 && y1 <> -1 && y2 <> -1) then begin
        isIt := true
    end;
    !isIt

(* ---------------------------------------------------------------------------
* REVERT
* ----------------------------------------------------------------------------
* DESCRIPTION : Reverse a list
*
* TYPE : Procedure
* ---------------------------------------------------------------------------*)

let revert mlist =
    let klist = ref [] in
    begin
        while mlist <> [] do
            match mlist with
            | [] -> klist := !klist;
            | e1 :: _ -> klist := e1 :: !klist;
        done;
    end;
    !klist


(* ---------------------------------------------------------------------------
* LINE2BLACK
* ----------------------------------------------------------------------------
* DESCRIPTION : Set all the pixel of a line to black
*
* TYPE : Procedure
*
* ARGUMENTS :
* matsrc : Input matrix
* y      : Line to modify
* ---------------------------------------------------------------------------*)

let line2black matsrc y =
        for i = 0 to (Array.length matsrc - 1) do
                matsrc.(i).(y) <- 1
        done

(* ---------------------------------------------------------------------------
* COLUMN2BLACK
* ----------------------------------------------------------------------------
* DESCRIPTION : Color a column into black
*
* TYPE : Procedure
*
* ARGUMENTS :
* matrix : Input matrix
* x      : x coordinate of the column
* y1     : Top of the column
* y2     : Bottom of the column
* ---------------------------------------------------------------------------*)

let column2black matrix x y1 y2 =
	for j = y1 to (y2) do
	    matrix.(x).(j) <- 1
	done

(* ---------------------------------------------------------------------------
* THEPIXEL
* ----------------------------------------------------------------------------
* DESCRIPTION : Calculates a percentage of black pixel into an area
*
* TYPE : Function, returns an int
* ---------------------------------------------------------------------------*)

let thePixel matrix =
    let cpt = ref 0. in

		for i = 0 to (width matrix) - 1 do

                        for j = 0 to (height matrix) - 1 do
                            if matrix.(i).(j) = 1 then
                                cpt := !cpt +. 1.
			done
		done;

                if (!cpt /. (float_of_int (size matrix))) < 0.5 then
		    cpt := 0.
		else
                    cpt := 1.;

                int_of_float(!cpt);;



(* ///////////////// END TOOLS ////////////////// *)





(* ---------------------------------------------------------------------------
* SEARCH_LINES
* ----------------------------------------------------------------------------
* DESCRIPTION : Analyses all the lines searching for black
*
* TYPE : Function. Returns bool array
*
* ARGUMENTS :
* matsrc : Input matrix (int matrix)
* ---------------------------------------------------------------------------*)

let search_lines matsrc =
        let tab = Array.make (Array.length matsrc.(0)) false in

        (* Browsing Width  *)
        for j = 0 to (Array.length matsrc.(0) - 1) do
                (* Browsing Height  *)
            for i = 0 to (Array.length matsrc - 1) do
                        if (matsrc.(i).(j) = 0) then
                                tab.(j) <- true
            done
        done;
        tab

(* ---------------------------------------------------------------------------
* SEARCH_LINES_V2
* ----------------------------------------------------------------------------
* DESCRIPTION : Creates the list of lines
*
* TYPE : Function. Returns a list of y-coordinates of lines
*
* ARGUMENTS :
* matrix    : Input matrix
* ---------------------------------------------------------------------------*)

let search_lines_v2 matrix =
	let mlist = ref [] in
	for j = 0 to (Array.length matrix.(0) - 1) do
	  if (matrix.(0).(j) = 1) then
              begin
                  mlist := !mlist @ [j] ;
	      end;
	done;
	!mlist

(* ---------------------------------------------------------------------------
* SEARCH_COLUMNS
* ----------------------------------------------------------------------------
* DESCRIPTION :
*
* TYPE : Function. Returns a bool array (true when there's black in the column)
*
* ARGUMENTS :
* matrix : Input matrix
* y1     : Top of the column
* y2     : Bottom of the column
* ---------------------------------------------------------------------------*)

let search_column matrix y1 y2=
	let tab = Array.make (Array.length matrix) false in
        for i = 0 to (Array.length matrix - 1) do
	  for j = y1 to (y2 - 1) do
	    if (matrix.(i).(j) = 0) then begin
	      tab.(i) <- true;
	    end
	  done
	done;
	tab

(* ---------------------------------------------------------------------------
* PRINT_LINES
* ----------------------------------------------------------------------------
* DESCRIPTION : Print black lines
*
* TYPE : Procedure
*
* ARGUMENTS :
* matrix : Input matrix
* ---------------------------------------------------------------------------*)

let print_lines matrix =
        let tab = search_lines matrix in
	let list_of_line = ref [] in
        for j = 0 to Array.length tab -2 do
                if (tab.(j+1) && not tab.(j)) then begin
                        line2black matrix (j);
			list_of_line := j:: !list_of_line;
                end
                else if (not tab.(j+1) && tab.(j)) then begin
                        line2black matrix (j+1);
			list_of_line := j:: !list_of_line;
                end
        done

(* ---------------------------------------------------------------------------
* PRINT_COLUMNS
* ----------------------------------------------------------------------------
* DESCRIPTION : Print black columns
*
* TYPE : Procedure
*
* ARGUMENTS :
* matrix : Input matrix
* y1     : Top of the column
* y2     : Bottom of the column
* ---------------------------------------------------------------------------*)

let print_columns matrix y1 y2 =
	let tab = search_column matrix y1 y2  in

        for i = 0 to ((Array.length tab) -2) do
	    if (tab.(i+1) && not tab.(i)) then
	            column2black matrix (i) y1 y2
	    else if (not tab.(i+1) && tab.(i)) then
	            column2black matrix (i+1) y1 y2
	done


(* ---------------------------------------------------------------------------
* PRINT_COLUMNS_CALL
* ----------------------------------------------------------------------------
* DESCRIPTION : Call function for print_columns
*
* TYPE : Procedure
*
* ARGUMENTS :
* matrix    : Input matrix
* liste     : Bool list related to lines
* ---------------------------------------------------------------------------*)

let print_columns_call matrix liste =
	let nth = ref (List.length liste - 1) in

        while (!nth > 0) do
	  let y1 = List.nth liste !nth in
	  let y2 = List.nth liste (!nth - 1) in
	    print_columns matrix y2 y1;
	    nth := !nth - 2;
	done

(* ---------------------------------------------------------------------------
* MAKECUBE9
* ----------------------------------------------------------------------------
* DESCRIPTION : Transform a matrix into a square matrix
*
* TYPE : Function
*
* ARGUMENTS :
* matrix    : Input matrix
* ---------------------------------------------------------------------------*)

let makeCube9 matrix =
    let x = ref (width matrix) in
    let y = ref (height matrix) in
    let cptX = ref 0 in
    let cptY = ref 0 in
    while (!x < !y) do
        x := !x + 2;
        cptX := !cptX + 1
    done;
    if (!x <> !y) then
    begin
        x := !x - 1;
        cptX := !cptX - 1
    end;
    while ((!x mod 9) <>0) do
        x := !x + 1;
        y := !y + 1;
        if ((!x mod 2) = 0) then
        begin
            cptX := !cptX + 1;
            cptY := !cptY + 1

        end
    done;
    let newMat = (Array.make_matrix !x !y 0) in
        for i = 0 to (width matrix) - 1 do
            for j = 0 to (height matrix) - 1 do
                newMat.(i + !cptX).(j + !cptY) <- matrix.(i).(j)
            done
        done;
        newMat

(* ---------------------------------------------------------------------------
* NINEXNINE
* ----------------------------------------------------------------------------
* DESCRIPTION : Transform a x*9 X x*9 matrix into a 9*9 matrix
*
* TYPE : Function. Return int matrix
*
* ARGUMENTS :
* matrix    : Input Matrix
* ---------------------------------------------------------------------------*)

let nineXnine matrix =
    let larg = (width matrix) / 9 in
    let haut = (height matrix) / 9 in
    let newMat = (Array.make_matrix 9 9 0) in
    let matTemp = (Array.make_matrix larg haut 0) in
        for i = 0 to 8 do
            for j = 0 to 8 do
                for x = (i * larg) to ((i + 1) * larg) -1 do
                    for y = (j * haut) to ((j + 1) * haut) -1 do
                        matTemp.(x - (i*larg)).(y - (j*haut)) <- matrix.(x).(y)
                    done
                done;
                newMat.(i).(j) <- thePixel matTemp
            done
        done;
        newMat

(* ---------------------------------------------------------------------------
* CREATE
* ----------------------------------------------------------------------------
* DESCRIPTION : Extracts a matrix from another bigger matrix
*
* TYPE : Function, returns an int matrix
*
* ARGUMENTS :
* matrix :
* x1     : x coordinate of top left corner
* x2     : x coordinate of top right corner
* y1     : y coordinate of top left corner
* y2     : y coordinate of bottom left corner
* ---------------------------------------------------------------------------*)

let create matrix x1 x2 y1 y2 =
        let newMat = (Array.make_matrix (abs (x2-x1)) (abs (y2-y1)) 0) in
            for i = x1 to x2 - 1 do
                for j = y1 to y2 - 1 do
                    newMat.(i-x1).(j-y1) <- matrix.(i).(j)
                done
            done;
        newMat

(* ---------------------------------------------------------------------------
* EXTRACT_LINES
* ----------------------------------------------------------------------------
* DESCRIPTION : Cut the initial matrix into lines
*
* TYPE : Function. Returns a list of int matrix
* ---------------------------------------------------------------------------*)
let extract_lines matrix =
    let larg = width matrix in
    let liste = search_lines_v2 matrix in
    let mlist = ref [] in
    let cpt = ref 0 in

    while (!cpt < List.length liste - 1) do

        let y1 = List.nth liste !cpt in
        let y2 = List.nth liste (!cpt + 1) in
        let y = y2 - y1 in

        let newMat = Array.make_matrix larg y 0 in

        for i = 0 to (larg -1) do
            for j = y1 to y2 -1 do
                newMat.(i).(j-y1) <- matrix.(i).(j)
            done
        done;

        mlist := !mlist @ [newMat];
        cpt := !cpt + 2
    done;
    !mlist

(* ---------------------------------------------------------------------------
* EXTRACT_CARACT
* ----------------------------------------------------------------------------
* DESCRIPTION : Cut the line matrix into characters
*
* TYPE : Function. Returns a list of int matrix
* ---------------------------------------------------------------------------*)

let extract_caract mat =
    let carlist = ref [] in
    let x = ref (-1) in
    for i = 0 to (width mat -1) do
        if (mat.(i).(0) = 1) then
            begin
                if (!x = (-1)) then
                    x := i
                else
                    begin
                        carlist := !carlist @ [create mat !x i 0 (height mat)];
                        x := -1
                    end
            end
    done;
    !carlist

(* ---------------------------------------------------------------------------
* CHAR_FROM_LINES
* ----------------------------------------------------------------------------
* DESCRIPTION : Extracts chars from all the lines already found
*
* TYPE : Function. Return something fucking not understandable (int matrix list)
*
* ARGUMENTS :
* mlist : matrix (lines) list
* ---------------------------------------------------------------------------*)

let char_from_lines mlist =
    let carlist = ref [] in
    for i = 0 to (List.length mlist -1) do
        carlist := !carlist @  (extract_caract (List.nth mlist i))
    done;
    !carlist

(* ---------------------------------------------------------------------------
* CFL_CALL
* ----------------------------------------------------------------------------
* DESCRIPTION : Cut the initial matrix into lines
*
* TYPE : Function. Returns a list of int matrix
* ---------------------------------------------------------------------------*)

let cfl_call mat =
    let linelist = extract_lines mat in
    let finalList = ref [] in
    finalList := char_from_lines linelist;
    !finalList

(* ---------------------------------------------------------------------------
* SHOW_CHARS
* ----------------------------------------------------------------------------
* DESCRIPTION : Displays all the characters step by step
*
* TYPE : Procedure
* ---------------------------------------------------------------------------*)

let show_chars mlist display =
    for i = 0 to (List.length mlist -1) do
        begin
            Display.show (Matrix.tab2image (Matrix.to3coord (List.nth mlist
            i))) display ;
            Display.wait_key ();
        end
    done
