(* Text bloc structure *)
type text_column =
{
    mutable left: int;
    mutable right: int;
}

type text_line =
{
    mutable top: int;
    mutable bottom: int;
    mutable height: int;
    mutable columns_list : text_column list;
}

(*
    --------------------
    | TEXT RECOGNITION |
    --------------------
*)

(* check if a pixel at a y height of the matrix
    is a line or not (ie it has one black pixel) *)
let isLine matsrc y =
    let is_line = ref false
    and x = ref 0 in
    while (!x <= (Array.length matsrc) - 1)
            && (not !is_line) do
        is_line := !is_line || (matsrc.(!x).(y) = (0,0,0));
        x := !x + 1
    done;
    !is_line;;

(* check if there is a black pixel in a line (ie between top and bottom
    of a line) (so it's a colmun)
    returns true if there is a black pixel in the column *)
let isColumn matsrc line x =
    let y = ref line.top
    and is_column = ref false in
    while (!y <= line.bottom - 1)
            && (not !is_column) do
        is_column := !is_column || (matsrc.(x).(!y) = (0,0,0));
        y := !y + 1;
    done;
    !is_column;;

(* detect_columns function finds the columns (which delimit the characters)
    whitin a line *)
let detect_columns matsrc line =
    let column_list = ref []
    and x = ref 0 in
    while (!x <= (Array.length matsrc) - 1) do
        if (isColumn matsrc line !x) then
            begin
                let new_column = {left = !x; right = !x; } in
                x := !x + 1;
                while (!x <= (Array.length matsrc) - 1)
                        && (isColumn matsrc line !x) do
                    x := !x + 1
                done;
                new_column.right <- !x;
                column_list := List.append (new_column::[]) !column_list;
            end;
        x := !x + 1
    done;
    !column_list;;

(* detect_line finds the lines in the matrix
    and add them in a list of lines *)
let detect_line matsrc =
    let line_list = ref [] in
    begin
        let line_top = ref 0
        and i = ref 0
        and j = ref 0 in
        while (!i <= (Array.length matsrc) - 1) do
            while (!j <= (Array.length matsrc.(0)) - 1) do
                if (matsrc.(!i).(!j) = (0,0,0)) then
                    begin
                        let line = {top = 0; bottom = 0;
                                    height = 0; columns_list = []; } in
                        line_top := !j;
                        j := !j + 1;
                        while (!j < (Array.length matsrc.(0)) - 1)
                                &&(isLine matsrc !j) do
                            j := !j + 1;
                        done;
                        line.top <- !line_top;
                        line.bottom <- !j;
                        line.height <- !j - !line_top;
                        line.columns_list <- detect_columns matsrc line;
                        line_list := List.append (line::[]) !line_list;
                    end;
                j := !j + 1
            done;
            i := !i + 1
        done;
    end;
    !line_list;;

(* print_columns function will print the left and right
    borders of the columns found in a line *)
let rec print_columns matsrc columns_list line_top line_bottom =
    let myMatrix = ref matsrc in
    match columns_list with
    |[]-> !myMatrix
    |column::l ->
                begin

                    for j = line_top to line_bottom - 1 do
                        matsrc.(column.right - 1).(j) <- (0,255,0);
                        matsrc.(column.left).(j) <- (0,255,0)
                    done;
                    myMatrix := print_columns !myMatrix l
                                                line_top line_bottom
                end;
                !myMatrix;;

(* print_lines function will print the top and bottom
    borders of the text lines founded *)
let rec print_lines matsrc lines_list =
    let myMatrix = ref matsrc in
    match lines_list with
    |[]-> !myMatrix
    |line::l ->
                begin
                    for i = 0 to (Array.length matsrc) - 1 do
                        matsrc.(i).(line.top) <- (255,0,0);
                        matsrc.(i).(line.bottom) <- (255,0,0)
                    done;
                    myMatrix := print_columns matsrc line.columns_list
                                                line.top line.bottom;
                    myMatrix := print_lines !myMatrix l
                end;
                !myMatrix;;

(* Main function for the textblock recognition
 *  recognise needs a properly filtered and binarised matrix
 *  display is used for displaying the image *)
let recognise matrix display =
    let myTextMatrix = print_lines matrix (detect_line (matrix)) in
        Display.show (Matrix.tab2image myTextMatrix) display

(* XY-cut for paragraph detection *)

(* horizontal histogram *)
let h_histogram mat top_line min_left =
    let hist_list = ref [] in
    let hist = ref 0 in
    for i = min_left to Array.length mat do
        hist := 0;
        for j = top_line to Array.length mat.(0) do
            if (mat.(i).(j) = (0,0,0)) then
                hist := !hist + 1
        done;
        hist_list := hist::(!hist_list)
    done;
    !hist_list


(* vertical histogram *)
let v_histogram mat top_line min_left =
    let hist_list = ref [] in
    let hist = ref 0 in
    for j = top_line to Array.length mat.(0) do
        hist := 0;
        for i = min_left to Array.length mat do
            if (mat.(i).(j) = (0,0,0)) then
                hist := !hist + 1
        done;
        hist_list := hist::(!hist_list)
    done;
    !hist_list
